// Cryptest.h : main header file for the CRYPTEST application
//

#if !defined(AFX_CRYPTEST_H__81EFC91B_30E4_4895_9246_4AECECDA381E__INCLUDED_)
#define AFX_CRYPTEST_H__81EFC91B_30E4_4895_9246_4AECECDA381E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"		// main symbols

/////////////////////////////////////////////////////////////////////////////
// CCryptestApp:
// See Cryptest.cpp for the implementation of this class
//

class CCryptestApp : public CWinApp
{
public:
	CCryptestApp();

	void Split(CString Source, CString Deliminator, CStringArray& AddIt, BOOL bAddEmpty);

	/* 
	Generate RSA Key
    parm1 = Key Length.
    parm1 = Private Key Filename.
    parm2 = Public Key Filename
    parm1 = Key Seed.
	returns = Nothing
	*/
	void GenerateRSAKey(unsigned int keyLength, const char *privFilename, const char *pubFilename, const char *seed);
	
	/* 
	Decrypt a String of data
    parm1 = Private Key File Location.
    parm2 = String to Decrypt
	returns = (CString) the Decrypted String
	*/
	CString RSADecryptString(const char *privFilename, const char *ciphertext);

	/* 
	Encrypt a String of data (using DES encryption)
    parm1 = Private Key File Location.
    parm2 = String to Decrypt
	returns = (CString) the Decrypted String
	*/
	CString RSAEncryptString(const char *pubFilename, const char *seed, const char *message);

	/* 
	Sign RSA File
    parm1 = Private Key Filename.
    parm2 = Message Filename.
    parm3 = Signature Filename
	returns = Nothing
	*/
	void RSASignFile(const char *privFilename, const char *messageFilename, const char *signatureFilename);

	/* 
	Verify RSA File
    parm1 = Public Key Filename.
    parm2 = Message Filename.
    parm3 = Signature Filename
	returns = bool
	*/
	bool RSAVerifyFile(const char *pubFilename, const char *messageFilename, const char *signatureFilename);
	
	/* 
	Encrypt a String of data (using DES encryption)
    parm1 = String to Encrypt.
    parm2 = Pass Phrase.
	returns = (CString) the Encrypted String
	*/
	CString EncryptString(const char *instr, const char *passPhrase);

	/* 
	Decrypt a String of data (using DES encryption)
    parm1 = String to Decrypt.
    parm2 = Pass Phrase.
	returns = (CString) the Decrypted String
	*/
	CString DecryptString(const char *instr, const char *passPhrase);

	void EncryptFile(const char *in, const char *out, const char *passPhrase);
	void DecryptFile(const char *in, const char *out, const char *passPhrase);
	void SecretShareFile(int threshold, int nShares, const char *filename, const char *seed);
	void SecretRecoverFile(int threshold, const char *outFilename, char *const *inFilenames);
	void InformationDisperseFile(int threshold, int nShares, const char *filename);
	void InformationRecoverFile(int threshold, const char *outFilename, char *const *inFilenames);
	void GzipFile(const char *in, const char *out, int deflate_level);
	void GunzipFile(const char *in, const char *out);
	void DigestFile(const char *filename);
	void ForwardTcpPort(const char *sourcePortName, const char *destinationHost, const char *destinationPortName);

	
	CListCtrl*	m_lcFileStuff;
	CListCtrl*	m_lcValidate;
	CListCtrl*	m_lcBenchMark;
	CListCtrl*	m_lcBenchMarkOps;
	CListCtrl*	m_lcSockets;

	int m_nItemCnt;
	int m_nFSItemCnt;
	int m_nBMItemCnt;
	int m_nBMItemCntOps;
	int m_nTFItemCnt;

	CString m_csThroughPut;

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CCryptestApp)
	public:
	virtual BOOL InitInstance();
	//}}AFX_VIRTUAL

// Implementation

	//{{AFX_MSG(CCryptestApp)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_CRYPTEST_H__81EFC91B_30E4_4895_9246_4AECECDA381E__INCLUDED_)
