// validat1.cpp - written and placed in the public domain by Wei Dai

#include "stdafx.h"
#include "Cryptest.h"

#include "pch.h"

#include "files.h"
#include "hex.h"
#include "cbc.h"
#include "cbcmac.h"
#include "dmac.h"
#include "idea.h"
#include "des.h"
#include "rc2.h"
#include "arc4.h"
#include "rc5.h"
#include "blowfish.h"
#include "diamond.h"
#include "wake.h"
#include "3way.h"
#include "safer.h"
#include "gost.h"
#include "shark.h"
#include "cast.h"
#include "square.h"
#include "seal.h"
#include "rc6.h"
#include "mars.h"
#include "rijndael.h"
#include "twofish.h"
#include "serpent.h"
#include "skipjack.h"
#include "rng.h"
#include "osrng.h"
#include "zdeflate.h"

#include <stdlib.h>
#include <time.h>
#include <memory>
#include <iostream>
#include <iomanip>

#include "validate.h"

USING_NAMESPACE(CryptoPP)
USING_NAMESPACE(std)

bool ValidateAll()
{
	bool pass=TestSettings();
	pass=TestOS_RNG();

	pass=CRC32Validate() && pass;
	pass=Adler32Validate() && pass;
	pass=MD2Validate() && pass;
	pass=MD5Validate() && pass;
	pass=SHAValidate() && pass;
	pass=SHA2Validate() && pass;
	pass=HAVALValidate() && pass;
	pass=TigerValidate() && pass;
	pass=RIPEMDValidate() && pass;
	pass=PanamaValidate() && pass;

	pass=MD5MACValidate() && pass;
	pass=HMACValidate() && pass;
	pass=XMACCValidate() && pass;

	pass=DESValidate() && pass;
	pass=CipherModesValidate() && pass;
	pass=IDEAValidate() && pass;
	pass=SAFERValidate() && pass;
	pass=RC2Validate() && pass;
	pass=ARC4Validate() && pass;
	pass=RC5Validate() && pass;
	pass=BlowfishValidate() && pass;
	pass=Diamond2Validate() && pass;
	pass=ThreeWayValidate() && pass;
	pass=GOSTValidate() && pass;
	pass=SHARKValidate() && pass;
	pass=CASTValidate() && pass;
	pass=SquareValidate() && pass;
	pass=SKIPJACKValidate() && pass;
	pass=SEALValidate() && pass;
	pass=RC6Validate() && pass;
	pass=MARSValidate() && pass;
	pass=RijndaelValidate() && pass;
	pass=TwofishValidate() && pass;
	pass=SerpentValidate() && pass;

	pass=BBSValidate() && pass;
	pass=DHValidate() && pass;
	pass=MQVValidate() && pass;
	pass=RSAValidate() && pass;
	pass=ElGamalValidate() && pass;
	pass=NRValidate() && pass;
	pass=DSAValidate() && pass;
	pass=LUCValidate() && pass;
	pass=LUCDIFValidate() && pass;
	pass=LUCELGValidate() && pass;
	pass=XTRDHValidate() && pass;
	pass=RabinValidate() && pass;
	pass=RWValidate() && pass;
	pass=BlumGoldwasserValidate() && pass;
	pass=ECPValidate() && pass;
	pass=EC2NValidate() && pass;
	pass=ECDSAValidate() && pass;

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	if (pass)
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("All tests passed!"));
	else
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("Oops!  Not all tests passed.!"));

	return pass;
}

bool Validate(int alg)
{
	switch (alg)
	{
	case 1: return TestSettings();
	case 2: return TestOS_RNG();
	case 3: return MD5Validate();
	case 4: return SHAValidate();
	case 5: return DESValidate();
	case 6: return IDEAValidate();
	case 7: return ARC4Validate();
	case 8: return RC5Validate();
	case 9: return BlowfishValidate();
	case 10: return Diamond2Validate();
	case 11: return ThreeWayValidate();
	case 12: return BBSValidate();
	case 13: return DHValidate();
	case 14: return RSAValidate();
	case 15: return ElGamalValidate();
	case 16: return DSAValidate();
	case 17: return HAVALValidate();
	case 18: return SAFERValidate();
	case 19: return LUCValidate();
	case 20: return RabinValidate();
	case 21: return BlumGoldwasserValidate();
	case 22: return ECPValidate();
	case 23: return EC2NValidate();
	case 24: return MD5MACValidate();
	case 25: return GOSTValidate();
	case 26: return TigerValidate();
	case 27: return RIPEMDValidate();
	case 28: return HMACValidate();
	case 29: return XMACCValidate();
	case 30: return SHARKValidate();
	case 32: return LUCDIFValidate();
	case 33: return LUCELGValidate();
	case 34: return SEALValidate();
	case 35: return CASTValidate();
	case 36: return SquareValidate();
	case 37: return RC2Validate();
	case 38: return RC6Validate();
	case 39: return MARSValidate();
	case 40: return RWValidate();
	case 41: return MD2Validate();
	case 42: return NRValidate();
	case 43: return MQVValidate();
	case 44: return RijndaelValidate();
	case 45: return TwofishValidate();
	case 46: return SerpentValidate();
	case 47: return CipherModesValidate();
	case 48: return CRC32Validate();
	case 49: return ECDSAValidate();
	case 50: return XTRDHValidate();
	case 51: return SKIPJACKValidate();
	case 52: return SHA2Validate();
	case 53: return PanamaValidate();
	case 54: return Adler32Validate();
	default: return ValidateAll();
	}
}

bool TestSettings()
{
	bool pass = true;

	CString csAppend = _T("");
	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("Testing Settings..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	if (*(word32 *)"\x01\x02\x03\x04" == 0x04030201L)
	{
#ifdef IS_LITTLE_ENDIAN
		csAppend += _T("passed:  ");
#else
		csAppend += _T("FAILED:  ");
		pass = false;
#endif
		csAppend += _T("Your machine is little endian.:  ");
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");
	}
	else if (*(word32 *)"\x01\x02\x03\x04" == 0x01020304L)
	{
#ifndef IS_LITTLE_ENDIAN
		csAppend += _T("passed:  ");
#else
		csAppend += _T("FAILED:  ");
		pass = false;
#endif
		csAppend += _T("Your machine is big endian.:  ");
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
	}
	else
	{
		csAppend += _T("FAILED:  Your machine is neither big endian nor little endian.");
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		pass = false;
	}
	csAppend = _T("");

	if (sizeof(byte) == 1)
		csAppend += _T("passed:  ");
	else
	{
		csAppend += _T("FAILED:  ");
		pass = false;
	}
	csFormat.Format("%ssizeof(byte) == %u", csAppend, sizeof(byte));
	csAppend = csFormat;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
	pcCryptest->m_nItemCnt++;
	csAppend = _T("");

	if (sizeof(word16) == 2)
		csAppend += _T("passed:  ");
	else
	{
		csAppend += _T("FAILED:  ");
		pass = false;
	}
	csFormat.Format("%ssizeof(word16) == %u", csAppend, sizeof(word16));
	csAppend = csFormat;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
	pcCryptest->m_nItemCnt++;
	csAppend = _T("");

	if (sizeof(word32) == 4)
		csAppend += _T("passed:  ");
	else
	{
		csAppend += _T("FAILED:  ");
		pass = false;
	}
	csFormat.Format("%ssizeof(word32) == %u", csAppend, sizeof(word32));
	csAppend = csFormat;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
	pcCryptest->m_nItemCnt++;
	csAppend = _T("");

#ifdef WORD64_AVAILABLE
	if (sizeof(word64) == 8)
		csAppend += _T("passed:  ");
	else
	{
		csAppend += _T("FAILED:  ");
		pass = false;
	}
	csFormat.Format("%ssizeof(word64) == %u", csAppend, sizeof(word64));
	csAppend = csFormat;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
	pcCryptest->m_nItemCnt++;
	csAppend = _T("");
#else
	if (sizeof(dword) >= 8)
	{
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("FAILED:  sizeof(dword) >= 8, but WORD64_AVAILABLE not defined"));
		pcCryptest->m_nItemCnt++;
		pass = false;
	}
	else
	{
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("passed:  word64 not available"));
		pcCryptest->m_nItemCnt++;
	}
#endif



	if (sizeof(dword) == 2*sizeof(word))
		csAppend += _T("passed:  ");
	else
	{
		csAppend += _T("FAILED:  ");
		pass = false;
	}
	csFormat.Format("%ssizeof(word) == %u, sizeof(dword) == %u", csAppend, sizeof(word), sizeof(dword));
	csAppend = csFormat;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
	pcCryptest->m_nItemCnt++;
	csAppend = _T("");

	dword test = (dword(1)<<WORD_BITS) + 2;
	if (HIGH_WORD(test) == 1 && LOW_WORD(test) == 2)
		csAppend += _T("passed:  ");
	else
	{
		csAppend += _T("FAILED:  ");
		pass = false;
	}
	csAppend += _T("HIGH_WORD() and LOW_WORD() macros");
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
	pcCryptest->m_nItemCnt++;
	csAppend = _T("");

	if (!pass)
	{
		csAppend += _T("Some critical setting in config.h is in error.  Please fix it and recompile.");
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");
		abort();
	}
	csAppend = _T("");
	return pass;
}

bool TestOS_RNG()
{
	bool pass = true;

	CString csAppend = _T("");
	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

#ifdef BLOCKING_RNG_AVAILABLE
	{
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
		pcCryptest->m_nItemCnt++;
		csAppend = _T("Testing operating system provided blocking random number generator...");
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");
//		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
//		pcCryptest->m_nItemCnt++;


		BlockingRng rng;
		ArraySink *sink;
		RandomNumberSource test(rng, 100000, false, new Deflator(sink=new ArraySink(NULL,0)));
		unsigned long total=0, length=0;
		time_t t;

		// check that it doesn't take too long to generate a reasonable amount of randomness
		t = time(NULL);
		test.Pump(16);
		total += 16;
		t = time(NULL) - t;
		if (t > 120)
		{
			csAppend += _T("FAILED:");
			pass = false;
		}
		else
			csAppend += _T("passed:");

		csFormat.Format("%s  it took %u seconds to generate 16 bytes", csAppend, t);
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");

		if (t < 2)
		{
			// that was fast, are we really blocking?
			// first exhaust the extropy reserve
			t = time(NULL);
			while (time(NULL) - t < 2)
			{
				test.Pump(8);
				total += 8;
			}

			// if it generates too many bytes in a certain amount of time,
			// something's probably wrong
			t = time(NULL);
			while (time(NULL) - t < 2)
			{
				test.Pump(8);
				total += 8;
				length += 8;
			}
			if (length > 1024)
			{
				csAppend += _T("FAILED:");
				pass = false;
			}
			else
				csAppend += _T("passed:");

			csFormat.Format("%s  it generated %u bytes in %u seconds", csAppend, length, time(NULL) - t);
			csAppend = csFormat;
			pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
			pcCryptest->m_nItemCnt++;
//			pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
//			pcCryptest->m_nItemCnt++;
			csAppend = _T("");
		}

		test.AttachedTransformation()->MessageEnd();

		if (sink->TotalPutLength() < total)
		{
			csAppend += _T("FAILED:");
			pass = false;
		}
		else
			csAppend += _T("passed:");

		csFormat.Format("%s  %u generated bytes compressed to %u bytes by DEFLATE", csAppend, total, sink->TotalPutLength());
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
//		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
//		pcCryptest->m_nItemCnt++;
		csAppend = _T("");
	}
#else
	csAppend = _T("No operating system provided blocking random number generator, skipping test.");
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
	pcCryptest->m_nItemCnt++;
//	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
//	pcCryptest->m_nItemCnt++;
	csAppend = _T("");

#endif

#ifdef NONBLOCKING_RNG_AVAILABLE
	{
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
		pcCryptest->m_nItemCnt++;
		csAppend = _T("Testing operating system provided nonblocking random number generator...");
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");

		NonblockingRng rng;
		ArraySink *sink = NULL;
		RandomNumberSource test(rng, 100000, true, new Deflator(sink=new ArraySink(NULL, 0)));
		
		if (sink->TotalPutLength() < 100000)
		{
			csAppend += _T("FAILED:");
			pass = false;
		}
		else
			csAppend += _T("passed:");

		csFormat.Format("%s  100000 generated bytes compressed to %u bytes by DEFLATE", csAppend, sink->TotalPutLength());
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
//		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
//		pcCryptest->m_nItemCnt++;
		csAppend = _T("");
	}
#else
	csAppend = _T("No operating system provided nonblocking random number generator, skipping test.");
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
	pcCryptest->m_nItemCnt++;
//	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
//	pcCryptest->m_nItemCnt++;
	csAppend = _T("");
#endif

	return pass;
}

// VC50 workaround
typedef auto_ptr<BlockTransformation> apbt;

class CipherFactory
{
public:
	virtual unsigned int BlockSize() const =0;
	virtual unsigned int KeyLength() const =0;

	virtual apbt NewEncryption(const byte *key) const =0;
	virtual apbt NewDecryption(const byte *key) const =0;
};

template <class E, class D> class FixedRoundsCipherFactory : public CipherFactory
{
public:
	FixedRoundsCipherFactory(unsigned int keylen=0) : m_keylen(keylen?keylen:E::DEFAULT_KEYLENGTH)
		{assert(E::KeyLength(m_keylen)==m_keylen && D::KeyLength(m_keylen)==m_keylen);}
	unsigned int BlockSize() const {return E::BLOCKSIZE;}
	unsigned int KeyLength() const {return m_keylen;}

	apbt NewEncryption(const byte *key) const
		{return apbt(new E(key, m_keylen));}
	apbt NewDecryption(const byte *key) const
		{return apbt(new D(key, m_keylen));}

	unsigned int m_keylen;
};

template <class E, class D> class VariableRoundsCipherFactory : public CipherFactory
{
public:
	VariableRoundsCipherFactory(unsigned int keylen=0, unsigned int rounds=0) : m_keylen(keylen?keylen:E::DEFAULT_KEYLENGTH), m_rounds(rounds?rounds:E::DEFAULT_ROUNDS)
		{assert(E::KeyLength(m_keylen)==m_keylen && D::KeyLength(m_keylen)==m_keylen);}
	unsigned int BlockSize() const {return E::BLOCKSIZE;}
	unsigned int KeyLength() const {return m_keylen;}

	apbt NewEncryption(const byte *key) const
		{return apbt(new E(key, m_keylen, m_rounds));}
	apbt NewDecryption(const byte *key) const
		{return apbt(new D(key, m_keylen, m_rounds));}

	unsigned int m_keylen, m_rounds;
};

bool BlockTransformationTest(const CipherFactory &cg, BufferedTransformation &valdata, unsigned int tuples = 0xffff)
{
	HexEncoder output(new FileSink(cout));
	SecByteBlock plain(cg.BlockSize()), cipher(cg.BlockSize()), out(cg.BlockSize()), outplain(cg.BlockSize());
	SecByteBlock key(cg.KeyLength());
	bool pass=true, fail;

	while (valdata.MaxRetrievable() && tuples--)
	{
		valdata.Get(key, cg.KeyLength());
		valdata.Get(plain, cg.BlockSize());
		valdata.Get(cipher, cg.BlockSize());

		apbt transE = cg.NewEncryption(key);
		transE->ProcessBlock(plain, out);
		fail = memcmp(out, cipher, cg.BlockSize()) != 0;

		apbt transD = cg.NewDecryption(key);
		transD->ProcessBlock(out, outplain);
		fail=fail || memcmp(outplain, plain, cg.BlockSize());

		pass = pass && !fail;

		CString csAppend = _T("");
		CString csFormat = _T("");

		CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

		csFormat.Format("%s", (fail ? "FAILED   " : "passed   "));

		CString csTest;
		for (int n=0; n<cg.KeyLength(); n++)
		{
			csTest.Format("%02x", key[n]);
			csTest.MakeUpper();
			csFormat += csTest;
		}
		csTest = _T("");
		csFormat += _T("   ");

		for (n=0; n<cg.BlockSize(); n++)
		{
			csTest.Format("%02x", outplain[n]);
			csTest.MakeUpper();
			csFormat += csTest;
		}
		csTest = _T("");
		csFormat += _T("   ");

		for (n=0; n<cg.BlockSize(); n++)
		{
			csTest.Format("%02x", out[n]);
			csTest.MakeUpper();
			csFormat += csTest;
		}
		csTest = _T("");

		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;

	}
	return pass;
}

class FilterTester : public Sink
{
public:
	FilterTester(const byte *validOutput, unsigned int outputLen)
		: validOutput(validOutput), outputLen(outputLen), counter(0), fail(false) {}
	void Put(byte inByte)
	{
		if (counter >= outputLen || validOutput[counter] != inByte)
			fail = true;
		counter++;
	}
	void Put(const byte *inString, unsigned int len)
	{
		while (len--)
			Put(*inString++);
	}
	void MessageEnd(int)
	{
		if (counter != outputLen)
			fail = true;
	}
	unsigned int Get(byte &outByte)
	{
		outByte = !fail;
		return 1;
	}

	const byte *validOutput;
	unsigned int outputLen, counter;
	bool fail;
};

bool TestFilter(BufferedTransformation &bt, const byte *in, unsigned int inLen, const byte *out, unsigned int outLen)
{
	bt.Attach(new FilterTester(out, outLen));
	LC_RNG rng(8595);
	while (inLen)
	{
		unsigned int randomLen = rng.GetLong(0, inLen);
		bt.Put(in, randomLen);
		in += randomLen;
		inLen -= randomLen;
	}
	bt.MessageEnd();
	byte result = 0;
	bt.Get(result);
	return result == 1;
}

bool DESValidate()
{
	CString csAppend = _T("");
//	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("DES validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	FileSource valdata("descert.dat", true, new HexDecoder);
	bool pass = BlockTransformationTest(FixedRoundsCipherFactory<DESEncryption, DESDecryption>(), valdata);

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("Testing EDE2, EDE3, and XEX3 variants..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;


	FileSource valdata1("3desval.dat", true, new HexDecoder);
	pass = BlockTransformationTest(FixedRoundsCipherFactory<DES_EDE2_Encryption, DES_EDE2_Decryption>(), valdata1, 1) && pass;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<DES_EDE3_Encryption, DES_EDE3_Decryption>(), valdata1, 1) && pass;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<DES_XEX3_Encryption, DES_XEX3_Decryption>(), valdata1, 1) && pass;

	return pass;
}

bool CipherModesValidate()
{
	CString csAppend = _T("");
	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("Testing DES modes..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	const byte key[] = {0x01,0x23,0x45,0x67,0x89,0xab,0xcd,0xef};
	const byte iv[] = {0x12,0x34,0x56,0x78,0x90,0xab,0xcd,0xef};
	const byte plain[] = {	// "Now is the time for all " without tailing 0
		0x4e,0x6f,0x77,0x20,0x69,0x73,0x20,0x74,
		0x68,0x65,0x20,0x74,0x69,0x6d,0x65,0x20,
		0x66,0x6f,0x72,0x20,0x61,0x6c,0x6c,0x20};
	DESEncryption desE(key);
	DESDecryption desD(key);
	bool pass=true, fail;

	{
		// from FIPS 81
		const byte encrypted[] = {
			0xE5, 0xC7, 0xCD, 0xDE, 0x87, 0x2B, 0xF2, 0x7C, 
			0x43, 0xE9, 0x34, 0x00, 0x8C, 0x38, 0x9C, 0x0F, 
			0x68, 0x37, 0x88, 0x49, 0x9A, 0x7C, 0x05, 0xF6};

		CBCRawEncryptor cbcE(desE, iv);
		fail = !TestFilter(cbcE, plain, sizeof(plain), encrypted, sizeof(encrypted));
		pass = pass && !fail;

		csFormat.Format("%sCBC encryption", (fail ? "FAILED   " : "passed   "));
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");
		
		CBCRawDecryptor cbcD(desD, iv);
		fail = !TestFilter(cbcD, encrypted, sizeof(encrypted), plain, sizeof(plain));
		pass = pass && !fail;

		csFormat.Format("%sCBC decryption", (fail ? "FAILED   " : "passed   "));
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");

	}
	{
		// generated with Crypto++, matches FIPS 81
		// but has extra 8 bytes as result of padding
		const byte encrypted[] = {
			0xE5, 0xC7, 0xCD, 0xDE, 0x87, 0x2B, 0xF2, 0x7C, 
			0x43, 0xE9, 0x34, 0x00, 0x8C, 0x38, 0x9C, 0x0F, 
			0x68, 0x37, 0x88, 0x49, 0x9A, 0x7C, 0x05, 0xF6, 
			0x62, 0xC1, 0x6A, 0x27, 0xE4, 0xFC, 0xF2, 0x77};

		CBCPaddedEncryptor cbcE(desE, iv);
		fail = !TestFilter(cbcE, plain, sizeof(plain), encrypted, sizeof(encrypted));
		pass = pass && !fail;

		csFormat.Format("%sCBC encryption with padding", (fail ? "FAILED   " : "passed   "));
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");

		CBCPaddedDecryptor cbcD(desD, iv);
		fail = !TestFilter(cbcD, encrypted, sizeof(encrypted), plain, sizeof(plain));
		pass = pass && !fail;

		csFormat.Format("%sCBC decryption with padding", (fail ? "FAILED   " : "passed   "));
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");

	}
	{
		// generated with Crypto++, matches FIPS 81
		// but with last two blocks swapped as result of CTS
		const byte encrypted[] = {
			0xE5, 0xC7, 0xCD, 0xDE, 0x87, 0x2B, 0xF2, 0x7C, 
			0x68, 0x37, 0x88, 0x49, 0x9A, 0x7C, 0x05, 0xF6, 
			0x43, 0xE9, 0x34, 0x00, 0x8C, 0x38, 0x9C, 0x0F};

		CBC_CTS_Encryptor cbcE(desE, iv);
		fail = !TestFilter(cbcE, plain, sizeof(plain), encrypted, sizeof(encrypted));
		pass = pass && !fail;
		
		csFormat.Format("%sCBC encryption with ciphertext stealing (CTS)", (fail ? "FAILED   " : "passed   "));
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");

		CBC_CTS_Decryptor cbcD(desD, iv);
		fail = !TestFilter(cbcD, encrypted, sizeof(encrypted), plain, sizeof(plain));
		pass = pass && !fail;

		csFormat.Format("%sCBC decryption with ciphertext stealing (CTS)", (fail ? "FAILED   " : "passed   "));
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");

	}
	{
		// generated with Crypto++
		const byte decryptionIV[] = {0x4D, 0xD0, 0xAC, 0x8F, 0x47, 0xCF, 0x79, 0xCE};
		const byte encrypted[] = {0x12, 0x34, 0x56};

		byte mutableIV[8];
		memcpy(mutableIV, iv, 8);

		CBC_CTS_Encryptor cbcE(desE, mutableIV, NULL, true);
		fail = !TestFilter(cbcE, plain, 3, encrypted, sizeof(encrypted));
		fail = memcmp(mutableIV, decryptionIV, 8) != 0 || fail;
		pass = pass && !fail;
		
		csFormat.Format("%sCBC encryption with ciphertext and IV stealing", (fail ? "FAILED   " : "passed   "));
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");

		CBC_CTS_Decryptor cbcD(desD, mutableIV);
		fail = !TestFilter(cbcD, encrypted, sizeof(encrypted), plain, 3);
		pass = pass && !fail;

		csFormat.Format("%sCBC decryption with ciphertext and IV stealing", (fail ? "FAILED   " : "passed   "));
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");

	}
	{
		const byte encrypted[] = {	// from FIPS 81
			0xF3,0x09,0x62,0x49,0xC7,0xF4,0x6E,0x51,
			0xA6,0x9E,0x83,0x9B,0x1A,0x92,0xF7,0x84,
			0x03,0x46,0x71,0x33,0x89,0x8E,0xA6,0x22};

		CFBEncryption cfbE(desE, iv);
		StreamCipherFilter eFilter(cfbE);
		fail = !TestFilter(eFilter, plain, sizeof(plain), encrypted, sizeof(encrypted));
		pass = pass && !fail;

		csFormat.Format("%sCFB encryption", (fail ? "FAILED   " : "passed   "));
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");

		CFBDecryption cfbD(desE, iv);
		StreamCipherFilter dFilter(cfbD);
		fail = !TestFilter(dFilter, encrypted, sizeof(encrypted), plain, sizeof(plain));
		pass = pass && !fail;

		csFormat.Format("%sCFB decryption", (fail ? "FAILED   " : "passed   "));
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");

	}
	{
		const byte encrypted[] = {	// from Eric Young's libdes
			0xf3,0x09,0x62,0x49,0xc7,0xf4,0x6e,0x51,
			0x35,0xf2,0x4a,0x24,0x2e,0xeb,0x3d,0x3f,
			0x3d,0x6d,0x5b,0xe3,0x25,0x5a,0xf8,0xc3};

		OFB ofb(desE, iv);
		StreamCipherFilter filter(ofb);
		fail = !TestFilter(filter, plain, sizeof(plain), encrypted, sizeof(encrypted));
		pass = pass && !fail;

		csFormat.Format("%sOFB", (fail ? "FAILED   " : "passed   "));
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");

	}
	{
		const byte encrypted[] = {	// generated with Crypto++
			0xF3, 0x09, 0x62, 0x49, 0xC7, 0xF4, 0x6E, 0x51, 
			0x16, 0x3A, 0x8C, 0xA0, 0xFF, 0xC9, 0x4C, 0x27, 
			0xFA, 0x2F, 0x80, 0xF4, 0x80, 0xB8, 0x6F, 0x75};

		CounterMode cm(desE, iv);
		StreamCipherFilter filter(cm);
		fail = !TestFilter(filter, plain, sizeof(plain), encrypted, sizeof(encrypted));
		pass = pass && !fail;

		csFormat.Format("%sCounter Mode", (fail ? "FAILED   " : "passed   "));
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");

	}
	{
		const byte plain[] = {	// "7654321 Now is the time for "
			0x37, 0x36, 0x35, 0x34, 0x33, 0x32, 0x31, 0x20, 
			0x4e, 0x6f, 0x77, 0x20, 0x69, 0x73, 0x20, 0x74, 
			0x68, 0x65, 0x20, 0x74, 0x69, 0x6d, 0x65, 0x20, 
			0x66, 0x6f, 0x72, 0x20};
		const byte mac1[] = {	// from FIPS 113
			0xf1, 0xd3, 0x0f, 0x68, 0x49, 0x31, 0x2c, 0xa4};
		const byte mac2[] = {	// generated with Crypto++
			0x35, 0x80, 0xC5, 0xC4, 0x6B, 0x81, 0x24, 0xE2};

		CBC_MAC<DESEncryption> cbcmac(key);
		HashFilter cbcmacFilter(cbcmac);
		fail = !TestFilter(cbcmacFilter, plain, sizeof(plain), mac1, sizeof(mac1));
		pass = pass && !fail;

		csFormat.Format("%sCBC MAC", (fail ? "FAILED   " : "passed   "));
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");

		DMAC<DESEncryption> dmac(key);
		HashFilter dmacFilter(dmac);
		fail = !TestFilter(dmacFilter, plain, sizeof(plain), mac2, sizeof(mac2));
		pass = pass && !fail;

		csFormat.Format("%sDMAC", (fail ? "FAILED   " : "passed   "));
		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
		pcCryptest->m_nItemCnt++;
		csAppend = _T("");

	}

	return pass;
}

bool IDEAValidate()
{
	CString csAppend = _T("");
//	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

//	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
//	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("IDEA validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	FileSource valdata("ideaval.dat", true, new HexDecoder);
	return BlockTransformationTest(FixedRoundsCipherFactory<IDEAEncryption, IDEADecryption>(), valdata);
}

bool SAFERValidate()
{
	CString csAppend = _T("");
//	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("SAFER validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	FileSource valdata("saferval.dat", true, new HexDecoder);
	bool pass = true;
	pass = BlockTransformationTest(VariableRoundsCipherFactory<SAFER_K64_Encryption, SAFER_K64_Decryption>(), valdata, 4) && pass;
	pass = BlockTransformationTest(VariableRoundsCipherFactory<SAFER_K128_Encryption, SAFER_K128_Decryption>(16,12), valdata, 4) && pass;
	pass = BlockTransformationTest(VariableRoundsCipherFactory<SAFER_SK64_Encryption, SAFER_SK64_Decryption>(8,6), valdata, 4) && pass;
	pass = BlockTransformationTest(VariableRoundsCipherFactory<SAFER_SK128_Encryption, SAFER_SK128_Decryption>(), valdata, 4) && pass;
	return pass;
}

bool RC2Validate()
{
	CString csAppend = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("RC2 validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	FileSource valdata("rc2val.dat", true, new HexDecoder);
	HexEncoder output(new FileSink(cout));
	SecByteBlock plain(RC2Encryption::BLOCKSIZE), cipher(RC2Encryption::BLOCKSIZE), out(RC2Encryption::BLOCKSIZE), outplain(RC2Encryption::BLOCKSIZE);
	SecByteBlock key(128);
	bool pass=true, fail;

	while (valdata.MaxRetrievable())
	{
		byte keyLen, effectiveLen;

		valdata.Get(keyLen);
		valdata.Get(effectiveLen);
		valdata.Get(key, keyLen);
		valdata.Get(plain, RC2Encryption::BLOCKSIZE);
		valdata.Get(cipher, RC2Encryption::BLOCKSIZE);

		apbt transE(new RC2Encryption(key, keyLen, effectiveLen));
		transE->ProcessBlock(plain, out);
		fail = memcmp(out, cipher, RC2Encryption::BLOCKSIZE) != 0;

		apbt transD(new RC2Decryption(key, keyLen, effectiveLen));
		transD->ProcessBlock(out, outplain);
		fail=fail || memcmp(outplain, plain, RC2Encryption::BLOCKSIZE);

		pass = pass && !fail;

		CString csAppend = _T("");
		CString csFormat = _T("");

		CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

		csFormat.Format("%s", (fail ? "FAILED   " : "passed   "));

		CString csTest;
		for (int n=0; n<keyLen; n++)
		{
			csTest.Format("%02x", key[n]);
			csTest.MakeUpper();
			csFormat += csTest;
		}
		csTest = _T("");
		csFormat += _T("   ");

		for (n=0; n<RC2Encryption::BLOCKSIZE; n++)
		{
			csTest.Format("%02x", outplain[n]);
			csTest.MakeUpper();
			csFormat += csTest;
		}
		csTest = _T("");
		csFormat += _T("   ");

		for (n=0; n<RC2Encryption::BLOCKSIZE; n++)
		{
			csTest.Format("%02x", out[n]);
			csTest.MakeUpper();
			csFormat += csTest;
		}
		csTest = _T("");

		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;

	}
	return pass;
}

bool ARC4Validate()
{
	unsigned char Key0[] = {0x01,0x23,0x45,0x67,0x89,0xab,0xcd,0xef };
	unsigned char Input0[]={0x01,0x23,0x45,0x67,0x89,0xab,0xcd,0xef};
	unsigned char Output0[] = {0x75,0xb7,0x87,0x80,0x99,0xe0,0xc5,0x96};

	unsigned char Key1[]={0x01,0x23,0x45,0x67,0x89,0xab,0xcd,0xef};
	unsigned char Input1[]={0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	unsigned char Output1[]={0x74,0x94,0xc2,0xe7,0x10,0x4b,0x08,0x79};

	unsigned char Key2[]={0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	unsigned char Input2[]={0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	unsigned char Output2[]={0xde,0x18,0x89,0x41,0xa3,0x37,0x5d,0x3a};

	unsigned char Key3[]={0xef,0x01,0x23,0x45};
	unsigned char Input3[]={0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	unsigned char Output3[]={0xd6,0xa1,0x41,0xa7,0xec,0x3c,0x38,0xdf,0xbd,0x61};

	unsigned char Key4[]={ 0x01,0x23,0x45,0x67,0x89,0xab, 0xcd,0xef };
	unsigned char Input4[] =
	{0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01};
	unsigned char Output4[]= {
	0x75,0x95,0xc3,0xe6,0x11,0x4a,0x09,0x78,0x0c,0x4a,0xd4,
	0x52,0x33,0x8e,0x1f,0xfd,0x9a,0x1b,0xe9,0x49,0x8f,
	0x81,0x3d,0x76,0x53,0x34,0x49,0xb6,0x77,0x8d,0xca,
	0xd8,0xc7,0x8a,0x8d,0x2b,0xa9,0xac,0x66,0x08,0x5d,
	0x0e,0x53,0xd5,0x9c,0x26,0xc2,0xd1,0xc4,0x90,0xc1,
	0xeb,0xbe,0x0c,0xe6,0x6d,0x1b,0x6b,0x1b,0x13,0xb6,
	0xb9,0x19,0xb8,0x47,0xc2,0x5a,0x91,0x44,0x7a,0x95,
	0xe7,0x5e,0x4e,0xf1,0x67,0x79,0xcd,0xe8,0xbf,0x0a,
	0x95,0x85,0x0e,0x32,0xaf,0x96,0x89,0x44,0x4f,0xd3,
	0x77,0x10,0x8f,0x98,0xfd,0xcb,0xd4,0xe7,0x26,0x56,
	0x75,0x00,0x99,0x0b,0xcc,0x7e,0x0c,0xa3,0xc4,0xaa,
	0xa3,0x04,0xa3,0x87,0xd2,0x0f,0x3b,0x8f,0xbb,0xcd,
	0x42,0xa1,0xbd,0x31,0x1d,0x7a,0x43,0x03,0xdd,0xa5,
	0xab,0x07,0x88,0x96,0xae,0x80,0xc1,0x8b,0x0a,0xf6,
	0x6d,0xff,0x31,0x96,0x16,0xeb,0x78,0x4e,0x49,0x5a,
	0xd2,0xce,0x90,0xd7,0xf7,0x72,0xa8,0x17,0x47,0xb6,
	0x5f,0x62,0x09,0x3b,0x1e,0x0d,0xb9,0xe5,0xba,0x53,
	0x2f,0xaf,0xec,0x47,0x50,0x83,0x23,0xe6,0x71,0x32,
	0x7d,0xf9,0x44,0x44,0x32,0xcb,0x73,0x67,0xce,0xc8,
	0x2f,0x5d,0x44,0xc0,0xd0,0x0b,0x67,0xd6,0x50,0xa0,
	0x75,0xcd,0x4b,0x70,0xde,0xdd,0x77,0xeb,0x9b,0x10,
	0x23,0x1b,0x6b,0x5b,0x74,0x13,0x47,0x39,0x6d,0x62,
	0x89,0x74,0x21,0xd4,0x3d,0xf9,0xb4,0x2e,0x44,0x6e,
	0x35,0x8e,0x9c,0x11,0xa9,0xb2,0x18,0x4e,0xcb,0xef,
	0x0c,0xd8,0xe7,0xa8,0x77,0xef,0x96,0x8f,0x13,0x90,
	0xec,0x9b,0x3d,0x35,0xa5,0x58,0x5c,0xb0,0x09,0x29,
	0x0e,0x2f,0xcd,0xe7,0xb5,0xec,0x66,0xd9,0x08,0x4b,
	0xe4,0x40,0x55,0xa6,0x19,0xd9,0xdd,0x7f,0xc3,0x16,
	0x6f,0x94,0x87,0xf7,0xcb,0x27,0x29,0x12,0x42,0x64,
	0x45,0x99,0x85,0x14,0xc1,0x5d,0x53,0xa1,0x8c,0x86,
	0x4c,0xe3,0xa2,0xb7,0x55,0x57,0x93,0x98,0x81,0x26,
	0x52,0x0e,0xac,0xf2,0xe3,0x06,0x6e,0x23,0x0c,0x91,
	0xbe,0xe4,0xdd,0x53,0x04,0xf5,0xfd,0x04,0x05,0xb3,
	0x5b,0xd9,0x9c,0x73,0x13,0x5d,0x3d,0x9b,0xc3,0x35,
	0xee,0x04,0x9e,0xf6,0x9b,0x38,0x67,0xbf,0x2d,0x7b,
	0xd1,0xea,0xa5,0x95,0xd8,0xbf,0xc0,0x06,0x6f,0xf8,
	0xd3,0x15,0x09,0xeb,0x0c,0x6c,0xaa,0x00,0x6c,0x80,
	0x7a,0x62,0x3e,0xf8,0x4c,0x3d,0x33,0xc1,0x95,0xd2,
	0x3e,0xe3,0x20,0xc4,0x0d,0xe0,0x55,0x81,0x57,0xc8,
	0x22,0xd4,0xb8,0xc5,0x69,0xd8,0x49,0xae,0xd5,0x9d,
	0x4e,0x0f,0xd7,0xf3,0x79,0x58,0x6b,0x4b,0x7f,0xf6,
	0x84,0xed,0x6a,0x18,0x9f,0x74,0x86,0xd4,0x9b,0x9c,
	0x4b,0xad,0x9b,0xa2,0x4b,0x96,0xab,0xf9,0x24,0x37,
	0x2c,0x8a,0x8f,0xff,0xb1,0x0d,0x55,0x35,0x49,0x00,
	0xa7,0x7a,0x3d,0xb5,0xf2,0x05,0xe1,0xb9,0x9f,0xcd,
	0x86,0x60,0x86,0x3a,0x15,0x9a,0xd4,0xab,0xe4,0x0f,
	0xa4,0x89,0x34,0x16,0x3d,0xdd,0xe5,0x42,0xa6,0x58,
	0x55,0x40,0xfd,0x68,0x3c,0xbf,0xd8,0xc0,0x0f,0x12,
	0x12,0x9a,0x28,0x4d,0xea,0xcc,0x4c,0xde,0xfe,0x58,
	0xbe,0x71,0x37,0x54,0x1c,0x04,0x71,0x26,0xc8,0xd4,
	0x9e,0x27,0x55,0xab,0x18,0x1a,0xb7,0xe9,0x40,0xb0,
	0xc0};

	// VC60 workaround: auto_ptr lacks reset()
	member_ptr<ARC4> arc4;
	bool pass=true, fail;
	int i;

	CString csAppend = _T("");
	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("ARC4 validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	arc4.reset(new ARC4(Key0, sizeof(Key0)));
	arc4->ProcessString(Input0, sizeof(Input0));
	fail = memcmp(Input0, Output0, sizeof(Input0)) != 0;
	csFormat.Format("%sTest 0", (fail ? "FAILED   " : "passed   "));
	csAppend = csFormat;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
	pcCryptest->m_nItemCnt++;
	csAppend = _T("");
	pass = pass && !fail;

	arc4.reset(new ARC4(Key1, sizeof(Key1)));
	arc4->ProcessString(Key1, Input1, sizeof(Key1));
	fail = memcmp(Output1, Key1, sizeof(Key1)) != 0;
	csFormat.Format("%sTest 1", (fail ? "FAILED   " : "passed   "));
	csAppend = csFormat;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
	pcCryptest->m_nItemCnt++;
	csAppend = _T("");
	pass = pass && !fail;

	arc4.reset(new ARC4(Key2, sizeof(Key2)));
	for (i=0, fail=false; i<sizeof(Input2); i++)
		if (arc4->ProcessByte(Input2[i]) != Output2[i])
			fail = true;
	csFormat.Format("%sTest 2", (fail ? "FAILED   " : "passed   "));
	csAppend = csFormat;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
	pcCryptest->m_nItemCnt++;
	csAppend = _T("");
	pass = pass && !fail;

	arc4.reset(new ARC4(Key3, sizeof(Key3)));
	for (i=0, fail=false; i<sizeof(Input3); i++)
		if (arc4->ProcessByte(Input3[i]) != Output3[i])
			fail = true;
	csFormat.Format("%sTest 3", (fail ? "FAILED   " : "passed   "));
	csAppend = csFormat;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
	pcCryptest->m_nItemCnt++;
	csAppend = _T("");
	pass = pass && !fail;

	arc4.reset(new ARC4(Key4, sizeof(Key4)));
	for (i=0, fail=false; i<sizeof(Input4); i++)
		if (arc4->ProcessByte(Input4[i]) != Output4[i])
			fail = true;
	csFormat.Format("%sTest 4", (fail ? "FAILED   " : "passed   "));
	csAppend = csFormat;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
	pcCryptest->m_nItemCnt++;
	csAppend = _T("");
	pass = pass && !fail;

	return pass;
}

bool RC5Validate()
{
	CString csAppend = _T("");
//	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("RC5 validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;


	FileSource valdata("rc5val.dat", true, new HexDecoder);
	return BlockTransformationTest(VariableRoundsCipherFactory<RC5Encryption, RC5Decryption>(16, 12), valdata);
}

bool RC6Validate()
{
	CString csAppend = _T("");
//	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("RC6 validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	FileSource valdata("rc6val.dat", true, new HexDecoder);
	bool pass = true;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<RC6Encryption, RC6Decryption>(16), valdata, 2) && pass;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<RC6Encryption, RC6Decryption>(24), valdata, 2) && pass;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<RC6Encryption, RC6Decryption>(32), valdata, 2) && pass;
	return pass;
}

bool MARSValidate()
{
	CString csAppend = _T("");
//	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("MARS validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	FileSource valdata("marsval.dat", true, new HexDecoder);
	bool pass = true;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<MARSEncryption, MARSDecryption>(16), valdata, 4) && pass;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<MARSEncryption, MARSDecryption>(24), valdata, 3) && pass;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<MARSEncryption, MARSDecryption>(32), valdata, 2) && pass;
	return pass;
}

bool RijndaelValidate()
{
	CString csAppend = _T("");
//	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("Rijndael validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	FileSource valdata("rijndael.dat", true, new HexDecoder);
	bool pass = true;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<RijndaelEncryption, RijndaelDecryption>(16), valdata, 4) && pass;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<RijndaelEncryption, RijndaelDecryption>(24), valdata, 3) && pass;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<RijndaelEncryption, RijndaelDecryption>(32), valdata, 2) && pass;
	return pass;
}

bool TwofishValidate()
{
	CString csAppend = _T("");
//	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("Twofish validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	FileSource valdata("twofishv.dat", true, new HexDecoder);
	bool pass = true;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<TwofishEncryption, TwofishDecryption>(16), valdata, 4) && pass;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<TwofishEncryption, TwofishDecryption>(24), valdata, 3) && pass;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<TwofishEncryption, TwofishDecryption>(32), valdata, 2) && pass;
	return pass;
}

bool SerpentValidate()
{
	CString csAppend = _T("");
//	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("Serpent validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	FileSource valdata("serpentv.dat", true, new HexDecoder);
	bool pass = true;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<SerpentEncryption, SerpentDecryption>(16), valdata, 4) && pass;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<SerpentEncryption, SerpentDecryption>(24), valdata, 3) && pass;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<SerpentEncryption, SerpentDecryption>(32), valdata, 2) && pass;
	return pass;
}

bool BlowfishValidate()
{
	CString csAppend = _T("");
	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("Blowfish validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	HexEncoder output(new FileSink(cout));
	char *key[]={"abcdefghijklmnopqrstuvwxyz", "Who is John Galt?"};
	byte *plain[]={(byte *)"BLOWFISH", (byte *)"\xfe\xdc\xba\x98\x76\x54\x32\x10"};
	byte *cipher[]={(byte *)"\x32\x4e\xd0\xfe\xf4\x13\xa2\x03", (byte *)"\xcc\x91\x73\x2b\x80\x22\xf6\x84"};
	byte out[8], outplain[8];
	bool pass=true, fail;

	for (int i=0; i<2; i++)
	{
		BlowfishEncryption enc((byte *)key[i], strlen(key[i]));
		enc.ProcessBlock(plain[i], out);
		fail = memcmp(out, cipher[i], 8) != 0;

		BlowfishDecryption dec((byte *)key[i], strlen(key[i]));
		dec.ProcessBlock(cipher[i], outplain);
		fail = fail || memcmp(outplain, plain[i], 8);
		pass = pass && !fail;

		csFormat.Format("%s", (fail ? "FAILED   " : "passed   "));

		CString csTest = _T("");
		csTest.Format("\"%s\"", key[i]);
		csFormat += csTest;

		csTest = _T("");

		for (int j=0; j<(signed int)(30-strlen(key[i])); j++)
			csFormat += _T(" ");

		csTest = _T("");
		csFormat += _T("   ");

		for (int n=0; n<8; n++)
		{
			csTest.Format("%02x", outplain[n]);
			csTest.MakeUpper();
			csFormat += csTest;
		}
		csTest = _T("");
		csFormat += _T("   ");

		for (n=0; n<8; n++)
		{
			csTest.Format("%02x", out[n]);
			csTest.MakeUpper();
			csFormat += csTest;
		}
		csTest = _T("");

		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;

	}
	return pass;
}

bool Diamond2Validate()
{
	CString csAppend = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("Diamond2 validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	FileSource valdata("diamond.dat", true, new HexDecoder);
	HexEncoder output(new FileSink(cout));
	byte key[32], plain[16], cipher[16], out[16], outplain[16];
	byte blocksize, rounds, keysize;
	bool pass=true, fail;
	member_ptr<BlockTransformation> diamond;	// VC60 workaround: auto_ptr lacks reset

	while (valdata.MaxRetrievable() >= 1)
	{
		valdata.Get(blocksize);
		valdata.Get(rounds);
		valdata.Get(keysize);
		valdata.Get(key, keysize);
		valdata.Get(plain, blocksize);
		valdata.Get(cipher, blocksize);

		if (blocksize==16)
			diamond.reset(new Diamond2Encryption(key, keysize, rounds));
		else
			diamond.reset(new Diamond2LiteEncryption(key, keysize, rounds));

		diamond->ProcessBlock(plain, out);
		fail=memcmp(out, cipher, blocksize) != 0;

		if (blocksize==16)
			diamond.reset(new Diamond2Decryption(key, keysize, rounds));
		else
			diamond.reset(new Diamond2LiteDecryption(key, keysize, rounds));

		diamond->ProcessBlock(out, outplain);
		fail=fail || memcmp(outplain, plain, blocksize);

		pass = pass && !fail;

		CString csAppend = _T("");
		CString csFormat = _T("");

		CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

		csFormat.Format("%s", (fail ? "FAILED   " : "passed   "));

		CString csTest;
		for (int n=0; n<keysize; n++)
		{
			csTest.Format("%02x", key[n]);
			csTest.MakeUpper();
			csFormat += csTest;
		}
		csTest = _T("");
		csFormat += _T("   ");

		for (n=0; n<blocksize; n++)
		{
			csTest.Format("%02x", outplain[n]);
			csTest.MakeUpper();
			csFormat += csTest;
		}
		csTest = _T("");
		csFormat += _T("   ");

		for (n=0; n<blocksize; n++)
		{
			csTest.Format("%02x", out[n]);
			csTest.MakeUpper();
			csFormat += csTest;
		}
		csTest = _T("");

		csAppend = csFormat;
		pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csAppend);
		pcCryptest->m_nItemCnt++;

	}
	return pass;
}

bool ThreeWayValidate()
{
	CString csAppend = _T("");
//	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("3-WAY validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	FileSource valdata("3wayval.dat", true, new HexDecoder);
	return BlockTransformationTest(FixedRoundsCipherFactory<ThreeWayEncryption, ThreeWayDecryption>(), valdata);
}

bool GOSTValidate()
{
	CString csAppend = _T("");
//	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("GOST validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	FileSource valdata("gostval.dat", true, new HexDecoder);
	return BlockTransformationTest(FixedRoundsCipherFactory<GOSTEncryption, GOSTDecryption>(), valdata);
}

bool SHARKValidate()
{
	CString csAppend = _T("");
//	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("SHARK validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;


#ifdef WORD64_AVAILABLE
	FileSource valdata("sharkval.dat", true, new HexDecoder);
	return BlockTransformationTest(FixedRoundsCipherFactory<SHARKEncryption, SHARKDecryption>(), valdata);
#else
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("word64 not available, skipping SHARK validation."));
	pcCryptest->m_nItemCnt++;
	return true;
#endif
}

bool CASTValidate()
{
	bool pass = true;

	CString csAppend = _T("");
//	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("CAST-128 validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	FileSource val128("cast128v.dat", true, new HexDecoder);
	pass = BlockTransformationTest(FixedRoundsCipherFactory<CAST128Encryption, CAST128Decryption>(16), val128, 1) && pass;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<CAST128Encryption, CAST128Decryption>(10), val128, 1) && pass;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<CAST128Encryption, CAST128Decryption>(5), val128, 1) && pass;

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("CAST-256 validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	FileSource val256("cast256v.dat", true, new HexDecoder);
	pass = BlockTransformationTest(FixedRoundsCipherFactory<CAST256Encryption, CAST256Decryption>(16), val256, 1) && pass;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<CAST256Encryption, CAST256Decryption>(24), val256, 1) && pass;
	pass = BlockTransformationTest(FixedRoundsCipherFactory<CAST256Encryption, CAST256Decryption>(32), val256, 1) && pass;

	return pass;
}

bool SquareValidate()
{
	CString csAppend = _T("");
//	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("Square validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	FileSource valdata("squareva.dat", true, new HexDecoder);
	return BlockTransformationTest(FixedRoundsCipherFactory<SquareEncryption, SquareDecryption>(), valdata);
}

bool SKIPJACKValidate()
{
	CString csAppend = _T("");
//	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("SKIPJACK validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	FileSource valdata("skipjack.dat", true, new HexDecoder);
	return BlockTransformationTest(FixedRoundsCipherFactory<SKIPJACKEncryption, SKIPJACKDecryption>(), valdata);
}

bool SEALValidate()
{
	byte input[] = {0x37,0xa0,0x05,0x95,0x9b,0x84,0xc4,0x9c,0xa4,0xbe,0x1e,0x05,0x06,0x73,0x53,0x0f,0x5f,0xb0,0x97,0xfd,0xf6,0xa1,0x3f,0xbd,0x6c,0x2c,0xde,0xcd,0x81,0xfd,0xee,0x7c};
	byte output[32];
	byte key[]={0x67, 0x45, 0x23, 0x01, 0xef, 0xcd, 0xab, 0x89, 0x98, 0xba, 0xdc, 0xfe, 0x10, 0x32, 0x54, 0x76, 0xc3, 0xd2, 0xe1, 0xf0};
	word32 start = 0x013577af;

	CString csAppend = _T("");
	CString csFormat = _T("");

	CCryptestApp* pcCryptest = (CCryptestApp*) AfxGetApp();

	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T("SEAL validation suite running..."));
	pcCryptest->m_nItemCnt++;
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, _T(""));
	pcCryptest->m_nItemCnt++;

	SEAL seal(key, start);
	unsigned int size = sizeof(input);
	bool pass = true;

	memset(output, 1, size);
	seal.ProcessString(output, input, size);
	for (unsigned int i=0; i<size; i++)
		if (output[i] != 0)
			pass = false;

	seal.Seek(0);
	output[0] = seal.ProcessByte(output[0]);
	seal.ProcessString(output+1, size-1);
	pass = pass && memcmp(output, input, size) == 0;

	csFormat.Format("%s", (pass ? "passed   " : "FAILED   "));
	pcCryptest->m_lcValidate->InsertItem(pcCryptest->m_nItemCnt, csFormat);
	pcCryptest->m_nItemCnt++;

	return pass;
}
